@@ -1,8 +1,7 @@
 '''Contains the DB wrapper class.'''
 from contextlib import (asynccontextmanager)
-from typing import (Any, AsyncIterator, Optional, List)
-from asyncio import (AbstractEventLoop)
-from logging import (Logger)
+from typing import (Any, AsyncIterator, Optional, List, cast)
+from logging import (Logger, getLogger)
 from asyncpg import (Connection, connect)
 import apgw.builder as builder
 from apgw.constraint import (Constraints)
@@ -10,21 +9,27 @@ from apgw.types import (ConnectionArgs, InsertAssignments, LimitOffset, Record,
 
 class DB:
   '''A simple wrapper around asyncpg to alleviate some boilerplate.'''
-  def __init__(self, conn_args: ConnectionArgs, loop: AbstractEventLoop, logger: Logger) -> None:
+  def __init__(
+      self,
+      conn_args: ConnectionArgs,
+      *,
+      auto_open: bool = False,
+      logger: Logger = getLogger('apgw'),
+  ) -> None:
     '''Initializes the DB instance.
 
     :param conn_args: the connection arguments
     :param logger: a Logger instance
     '''
     self.conn_args = conn_args
-    self.loop = loop
+    self.auto_open = auto_open
     self.logger = logger
     self._conn: Optional[Connection] = None
 
   @property
   def is_open(self) -> bool:
     '''Returns True if the instance is connected.'''
-    return bool(self.conn)
+    return bool(self._conn)
 
   @property
   def conn(self) -> Connection:
@@ -32,13 +37,18 @@ class DB:
       raise Exception('Connection is not open')
     return self._conn
 
+  @property
+  def _should_auto_open(self) -> bool:
+    '''Returns True if the connection can and should be opened automatically.'''
+    return self.auto_open and not self.is_open
+
   async def open(self) -> 'DB':
     '''Opens the connection.'''
     if not self._conn:
       if isinstance(self.conn_args, str):
-        self._conn = await connect(loop=self.loop, dsn=self.conn_args)
+        self._conn = await connect(dsn=self.conn_args)
       else:
-        self._conn = await connect(loop=self.loop, **self.conn_args)
+        self._conn = await connect(**self.conn_args)
     return self
 
   async def close(self) -> 'DB':
@@ -79,12 +89,15 @@ class DB:
 
     self._log(sql, params)
 
+    if self._should_auto_open:
+      await self.open()
+
     if params:
       return await self.conn.fetch(sql, *params)
 
     return await self.conn.fetch(sql)
 
-  async def fetchrow(self, sql: Optional[str], params: List[Any] = None) -> Record:
+  async def fetchrow(self, sql: Optional[str], params: List[Any] = None) -> Optional[Record]:
     '''This method passes through to self.conn.fetchrow.
 
     :param sql: the parameterized sql to execute
@@ -95,6 +108,9 @@ class DB:
 
     self._log(sql, params)
 
+    if self._should_auto_open:
+      await self.open()
+
     if params:
       return await self.conn.fetchrow(sql, *params)
 
@@ -144,12 +160,12 @@ class DB:
     :param order_by: how to order the results
     :param limit: the limit and offset of the results
     '''
-    count_record = await self.select_one(
+    count_record = cast(Record, await self.select_one(
         table_name,
         constraints,
         columns='count(*) as c',
         order_by=order_by,
-    )
+    ))
     records = await self.select(
         table_name,
         constraints,
@@ -157,7 +173,7 @@ class DB:
         order_by=order_by,
         limit=limit,
     )
-    return (records, count_record['c'])
+    return (records, cast(int, count_record['c']))
 
   async def select_one(
       self,
@@ -165,7 +181,7 @@ class DB:
       constraints: Constraints = None,
       *,
       columns: str = '*',
-      order_by: str = None) -> Record:
+      order_by: str = None) -> Optional[Record]:
     '''Selects a single record from the database.
 
     :param table_name: the name of the table to select from
@@ -210,7 +226,8 @@ class DB:
         assignments,
         returning=returning)
 
-    return await self.fetchrow(sql, params)
+    output = await self.fetchrow(sql, params)
+    return cast(Record, output)
 
   async def create(
       self,
